# Generated by Django 3.1.5 on 2021-01-06 03:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("lti_toolbox", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="lticonsumer",
            name="url",
            field=models.URLField(
                blank=True,
                help_text="URL of the LTI consumer website",
                max_length=1024,
                verbose_name="Consumer site URL",
            ),
        ),
    ]
