"""lti_toolbox is a django application to handle LTI Provider related
operations, like launch request verification and user authentication."""

# pylint: disable=invalid-name
default_app_config = "lti_toolbox.apps.LtiProviderAppConfig"
